require 'spec_helper'

describe Payzen::PaymentResponse do
  let(:params) do
    params = {vads_action_mode: 'INTERACTIVE',
      vads_ctx_mode: 'TEST',
      vads_contrib: 'Wizypay',
      vads_page_action: 'PAYMENT',
      vads_return_mode: 'POST',
      vads_version: 'V2',
      vads_currency: '978',
      vads_language: 'fr',
      vads_site_id: '84197771',
      vads_payment_config: 'SINGLE',
      vads_trans_date: '20150722135629',
      vads_trans_id: '741418',
      vads_amount: '5000',
      vads_order_id: '741418',
      vads_cust_id: '3000',
      signature: '8d56ef75763dec4bad4257a6e726cc1015129157'}
    Payzen.setup do |config|
    end
    params
  end

  let (:result_base) do
    result_base = {vads_action_mode: 'INTERACTIVE', vads_ctx_mode:'TEST',
    vads_contrib:'Wizypay', vads_page_action:'PAYMENT',
    vads_return_mode:'POST', vads_version:'V2',
    vads_currency:'978', vads_language:'fr',
    vads_site_id:'84197771', vads_payment_config:'SINGLE',
    vads_trans_date:'20150722135629', vads_trans_id:'741418',
    vads_amount:'5000', vads_order_id:'741418',
    vads_cust_id:'3000', vads_effective_amount:'5000'}
  end

  describe '#response_without_psp' do
    it 'retuns params if debit is empty' do
      result = Payzen::PaymentResponse.response_without_psp(params, '111111', 'Wizypay', [])
      expect(result).to eql params
    end

    it 'retuns a simple response if only one card has been used' do
      result = {signature:'7cbd634540357f418822840e3c15213d71398145',
      vads_presentation_date:'20150722135629',
      vads_effective_creation_date:'20150722135629', vads_warranty_result:'NO',
      vads_validation_mode:'0', vads_card_brand:'Wizypay',
      vads_card_number:'ABCD', vads_trans_status:'CAPTURED',
      vads_capture_delay:'0', vads_auth_mode:'FULL',
      vads_auth_number:'', vads_auth_result:'00',
      vads_threeds_enrolled:'', vads_threeds_status:'',
      vads_sequence_number:'1', vads_ext_trans_id:'1',
      vads_operation_type:'DEBIT', vads_payment_certificate:'',
      vads_card_country:'FR', vads_result:'00'}
      result = result_base.merge(result)
      expect(Payzen::PaymentResponse.response_without_psp(params, '1', 'Wizypay', [{'code'=> 'ABCD'}])).to eql result
    end

    it 'returns a fractional_payment if multiple debits' do
      result = {signature: "5c0621391fd667deb7f388d3c450a1ee3b560654",vads_card_brand: "MULTI",
      vads_payment_seq: "{\"trans_id\":\"741418\",\"transactions\":[{\"amount\":300000,\"operation_type\":\"DEBIT\",\"capture_delay\":\"0\",\"card_brand\":\"Wizypay\",\"card_number\":\"ABCD\",\"payment_certificate\":\"\",\"presentation_date\":\"20150722135629\",\"ext_trans_id\":\"1\",\"sequence_number\":1,\"trans_id\":\"741418\",\"trans_status\":\"CAPTURED\"},{\"amount\":200000,\"operation_type\":\"DEBIT\",\"capture_delay\":\"0\",\"card_brand\":\"Wizypay\",\"card_number\":\"EFGH\",\"payment_certificate\":\"\",\"presentation_date\":\"20150722135629\",\"ext_trans_id\":\"1\",\"sequence_number\":2,\"trans_id\":\"741418\",\"trans_status\":\"CAPTURED\"}]}"}
      result = result_base.merge(result)
      response = Payzen::PaymentResponse.response_without_psp(params, '1', 'Wizypay', [{'code'=> 'ABCD', 'amount'=> '3000'}, {'code'=>'EFGH', 'amount'=>'2000'}])
      expect(response).to eql result
    end

    it "returns CANCELED" do
      expect(Payzen::PaymentResponse.response_without_psp(params, '1', 'Wizypay', [{'code'=> 'ABCD'}], 'CANCELED')[:vads_trans_status]).to eql 'CANCELED'
    end
  end

  describe "#response_with_psp" do
    it "returns params if debits is nil" do
      expect(Payzen::PaymentResponse.response_with_psp(params, '90000', 'Wizypay', nil, '4000')).to eql params
    end

    it "returns a fractional response" do
      result = {signature: "47fd7bb9ecdfc8cbbba23c1e639e95a8748963d3",
      vads_effective_amount: '10000', vads_amount: '10000',
      vads_operation_type: "DEBIT", vads_auth_number: "00",
      vads_capture_delay: "0", vads_card_brand: "MULTI",
      vads_payment_certificate: "1a2154678f456d44c1d54vs6488s",
      vads_trans_status: "AUTHORISED",
      vads_payment_seq: "{\"trans_id\":\"741418\",\"transactions\":[{\"amount\":2500,\"operation_type\":\"DEBIT\",\"capture_delay\":\"0\",\"card_brand\":\"Wizypay\",\"card_number\":\"ABCD\",\"payment_certificate\":\"\",\"presentation_date\":\"20150722135629\",\"ext_trans_id\":\"90000\",\"sequence_number\":1,\"trans_id\":\"741418\",\"trans_status\":\"CAPTURED\"},{\"amount\":2500,\"operation_type\":\"DEBIT\",\"capture_delay\":\"0\",\"card_brand\":\"Wizypay\",\"card_number\":\"EFGH\",\"payment_certificate\":\"\",\"presentation_date\":\"20150722135629\",\"ext_trans_id\":\"90000\",\"sequence_number\":2,\"trans_id\":\"741418\",\"trans_status\":\"CAPTURED\"},{\"amount\":\"5000\",\"operation_type\":\"DEBIT\",\"auth_number\":\"00\",\"capture_delay\":\"0\",\"card_brand\":\"CB\",\"card_number\":\"123456789\",\"expiry_month\":\"\",\"expiry_year\":\"\",\"payment_certificate\":\"1a2154678f456d44c1d54vs6488s\",\"presentation_date\":\"20150722135629\",\"trans_id\":\"741418\",\"trans_status\":\"AUTHORISED\",\"sequence_number\":3}]}"}
      payment = {vads_operation_type: 'DEBIT',
                vads_auth_number: '00',
                vads_capture_delay: '0',
                vads_card_brand: 'CB',
                vads_card_number: '123456789',
                vads_expiry_month: '',
                vads_expiry_year: '',
                vads_payment_certificate: '1a2154678f456d44c1d54vs6488s',
                vads_trans_status: 'AUTHORISED'}
      payment = params.merge(payment)
      result = result_base.merge(result)
      response = Payzen::PaymentResponse.response_with_psp(payment, '90000', 'Wizypay', [{'code'=> 'ABCD', 'amount'=> '25'}, {'code'=>'EFGH', 'amount'=>'25'}], '50')
      expect(response).to eql result
    end
  end
end
